From 993f164c5421a060ced2b0180982a18e63641208 Mon Sep 17 00:00:00 2001
From: Guillaume Tucker <gtucker@gtucker.io>
Date: Wed, 18 Sep 2024 06:24:01 +0200
Subject: [PATCH] [RFC] Makefile: wrap in container if CONTAINER is defined

Wrap the make command in a container if $(CONTAINER) is defined.  This
variable actually provides the image name.  Then $(CONTAINER_COMMAND)
is to invoke the container manager, 'docker' by default.

This is a proof-of-concept implementation to illustrate the idea.

For example, using a tuxmake image with a kernel.org toolchain:

$ make CONTAINER=tuxmake/korg-clang-18 LLVM=1 defconfig
Running in docker tuxmake/korg-clang-18
  HOSTCC  scripts/basic/fixdep
  HOSTCC  scripts/kconfig/conf.o
  HOSTCC  scripts/kconfig/confdata.o
  HOSTCC  scripts/kconfig/expr.o
  LEX     scripts/kconfig/lexer.lex.c
  YACC    scripts/kconfig/parser.tab.[ch]
  HOSTCC  scripts/kconfig/lexer.lex.o
  HOSTCC  scripts/kconfig/menu.o
  HOSTCC  scripts/kconfig/parser.tab.o
  HOSTCC  scripts/kconfig/preprocess.o
  HOSTCC  scripts/kconfig/symbol.o
  HOSTCC  scripts/kconfig/util.o
  HOSTLD  scripts/kconfig/conf
*** Default configuration is based on 'x86_64_defconfig'

$ make CONTAINER=tuxmake/korg-clang-18 LLVM=1 -j8
Running in docker tuxmake/korg-clang-18
make: warning: -j8 forced in submake: resetting jobserver mode.
  GEN     arch/x86/include/generated/asm/orc_hash.h
  WRAP    arch/x86/include/generated/uapi/asm/bpf_perf_event.h
  WRAP    arch/x86/include/generated/uapi/asm/fcntl.h
[...]
  MKPIGGY arch/x86/boot/compressed/piggy.S
  AS      arch/x86/boot/compressed/piggy.o
  LD      arch/x86/boot/compressed/vmlinux
  ZOFFSET arch/x86/boot/zoffset.h
  OBJCOPY arch/x86/boot/vmlinux.bin
  AS      arch/x86/boot/header.o
  LD      arch/x86/boot/setup.elf
  OBJCOPY arch/x86/boot/setup.bin
  BUILD   arch/x86/boot/bzImage
Kernel: arch/x86/boot/bzImage is ready  (#1)

Signed-off-by: Guillaume Tucker <gtucker@gtucker.io>
---
 Makefile | 12 ++++++++++++
 1 file changed, 12 insertions(+)

diff --git a/Makefile b/Makefile
index 34bd1d5f9672..33620b82697e 100644
--- a/Makefile
+++ b/Makefile
@@ -1,4 +1,15 @@
 # SPDX-License-Identifier: GPL-2.0
+ifneq ($(CONTAINER),)
+CONTAINER_COMMAND ?= docker
+PHONY := __all
+__all:
+%:
+	@echo Running in $(CONTAINER_COMMAND) $(CONTAINER)
+	@$(CONTAINER_COMMAND) run -it -v $(PWD):/src -w /src \
+	$(CONTAINER) $(MAKE) \
+	$(subst CONTAINER=$(CONTAINER),,$(MAKEFLAGS)) \
+	$(GNUMAKEFLAGS) $(MAKECMDGOALS)
+else
 VERSION = 6
 PATCHLEVEL = 11
 SUBLEVEL = 0
@@ -2070,3 +2081,4 @@ FORCE:
 # Declare the contents of the PHONY variable as phony.  We keep that
 # information in a variable so we can use it in if_changed and friends.
 .PHONY: $(PHONY)
+endif # CONTAINER
-- 
2.39.2

